/* written by Katy Perdue
   created on or before May 30, 2003   
   last modified June 11, 2003  */


/* support files*/

:- ensure_loaded(createScale).
:-ensure_loaded(parse).
:-ensure_loaded(startScale).
:-ensure_loaded(library(random)).

/*  PossScale2 is an artifact from preliminary work.  It uses parse
    to break understand what's been entered, then sends the information
    to suggScales to find scale suggestions.  The two functors here 
    work for chords entered with or without a start note.  Unfortunately,
    this structure is difficult to control, and therefore not terribly 
    useful.  */

possScale2(Chord, StartNote):-
    parse(Chord, Class, Accidental, Qual, _, _, _),
    suggScales(Class, Accidental, Qual, StartNote).

possScale2(Chord):-
    parse(Chord, Class, Accidental, Qual, _, _, _),
    suggScales(Class, Accidental, Qual).

/* possScale is designed so that the user does not have to enter in 
   a bunch of blanks in order for the computer to parse the chord.
   It simply sends the chord and the required number of blanks off to
   parse 2. */

possScale(Chord):-
    parse2(Chord, _, _, _, _, _).

possScale(Chord, StartNote):-
    parse2(Chord, _, _, _, _, _, StartNote).

/* parse2 both parses the user-entered chord, and suggests scales that go
   with that chord.  The functor is set up to accept either a single chord
   or a list of chords.  The auxilary functors are located in parse.pl */

parse2([Chord|[]], Dur, Class, Accidental, Qual, Ext):-
    parse2(Chord, Dur, Class, Accidental, Qual, Ext), !.

parse2([Chord|List], _, _, _,_,_):-
    parse2(Chord, _, _, _, _, _), nl,
    parse2(List, _,_, _, _, _).

parse2(Chord, Dur, Class, Accidental, Qual, ExtensionList):-
    write('Chord: '), write(Chord), nl,
    name(Chord, [PreDur|ChordAtoms]),
    Dur is -(PreDur,48),    class(ChordAtoms, Class),
    sharpOrFlat(ChordAtoms, [Q, U, A| L ],  Accidental),
    quality(Q, U, A, Qual),
    findall([ExtSign|Ext], extensions(L, [ExtSign|Ext]), ExtensionList),
    suggScales(Class, Accidental, Qual, ExtensionList).

/* The same as the parse2 above, though this version keeps track of a user-
   entered start note. */

parse2([Chord|[]], Dur, Class, Accidental, Qual, Ext, StartNote):-
    parse2(Chord, Dur, Class, Accidental, Qual, Ext,  StartNote), !.

parse2([Chord|List], _, _, _,_,_, StartNote):-
    parse2(Chord, _, _, _, _, _,  StartNote), nl,
    parse2(List, _,_, _, _, _, StartNote).

parse2(Chord, Dur, Class, Accidental, Qual, ExtensionList, StartNote):-
    write('Chord: '), write(Chord), nl,
    name(Chord, [PreDur|ChordAtoms]),
    Dur is -(PreDur,48),    class(ChordAtoms, Class),
    sharpOrFlat(ChordAtoms, [Q, U, A| L ],  Accidental),
    quality(Q, U, A, Qual),
    findall([ExtSign|Ext], extensions(L, [ExtSign|Ext]), ExtensionList),
    suggScales(Class, Accidental, Qual, ExtensionList, StartNote).

/* extensions is a form of extension (from the parse.pl file) designed
   to handle more than one extension (or none at all). */

extensions([Num, Acc|Rest], [ExtSign|Extension]):-
    extension(Num, Extension),
    sharpOrFlat(Acc, ExtSign).

extensions([_, _|TheRest], [ExtSign|Extension]):-
    extensions(TheRest, [ExtSign|Extension]).


/*If no start note is specified, this functor tells the scale to 
    start on the root*/

suggScales(Class, Accidental, Qual, ExtensionList):-
    scale(Class, Accidental, [Root|_]),
    suggScales(Class, Accidental, Qual, ExtensionList, Root).

/* suggScales work by first finding the quality of the scale.  It then
   looks for extensions.  It then generates a random number, and sends the
   information off to the appropriate write scale functor.  The random 
   number is used to determine which scale will be printed when there is 
   more than one possible scale.  The random number is multiplied by 10, 
   as I had trouble getting prolog to handle decimals. */

/*Possible major scales. */

suggScales(Class, Accidental, Qual, ExtensionList, StartNote):-
    Qual=='major',
    member([flat|' sixth'], ExtensionList),
    scale(Class, Accidental, ScaleNotes),
    random(X),
    maj6WriteScale(ScaleNotes, StartNote, X*10).

suggScales(Class, Accidental, Qual, ExtensionList, StartNote):-
    Qual=='major',
    member([sharp|' fourth'], ExtensionList),
    member([sharp|' fifth'], ExtensionList),
    scale(Class, Accidental, ScaleNotes),
    random(X),
    maj45WriteScale(ScaleNotes, StartNote, X*10).

suggScales(Class, Accidental, Qual, ExtensionList, StartNote):-
    Qual=='major',
    member([sharp|' fourth'], ExtensionList),
    scale(Class, Accidental, ScaleNotes),
    random(X),
    majSusWriteScale(ScaleNotes, StartNote, X*10).

suggScales(Class, Accidental, Qual, ExtensionList, StartNote):-
    Qual=='major',
    member([' '|' seventh'], ExtensionList),
    scale(Class, Accidental, ScaleNotes),
    random(X),
    maj7WriteScale(ScaleNotes, StartNote, X*10).
    
suggScales(Class, Accidental, Qual, _, StartNote):-
    Qual=='major',
    scale(Class, Accidental, ScaleNotes),
    random(X),
    majWriteScale(ScaleNotes, StartNote, X*10).

/*Possible diminished scale */

suggScales(Class, Accidental, Qual,_, StartNote):-
    Qual == 'diminished',
    scale(Class, Accidental, ScaleNotes),
    write('Diminished '), dim8(ScaleNotes, DimScale),
    startSc(DimScale, StartNote), nl.

/*Possible half diminished scales */

suggScales(Class, Accidental, Qual,ExtensionList, StartNote):-
    Qual =='locrian or half diminished',
    member([sharp|' second'], ExtensionList),
    scale(Class, Accidental, ScaleNotes),
    random(X),
    locSharp2WriteScale(ScaleNotes, StartNote, X*10).

suggScales(Class, Accidental, Qual,_, StartNote):-
    Qual =='locrian or half diminished',
    scale(Class, Accidental, ScaleNotes),
    random(X),
    locWriteScale(ScaleNotes, StartNote, X*10).

/*Possible minor scales */

suggScales(Class, Accidental, Qual,ExtensionList, StartNote):-
    Qual=='minor',
    member([flat|' ninth'], ExtensionList),
    member([flat|' sixth'], ExtensionList),
    scale(Class, Accidental, ScaleNotes),
    random(X),
    min9bWriteScale(ScaleNotes, StartNote, X*10).

suggScales(Class, Accidental, Qual,ExtensionList, StartNote):-
    Qual=='minor',
    member([flat|' sixth'], ExtensionList),
    scale(Class, Accidental, ScaleNotes),
    random(X),
    min6bWriteScale(ScaleNotes, StartNote, X*10).

suggScales(Class, Accidental, Qual,ExtensionList, StartNote):-
    Qual=='minor',
    member([sharp|' seventh'], ExtensionList),
    scale(Class, Accidental, ScaleNotes),
    random(X),
    minSharp7WriteScale(ScaleNotes, StartNote, X*10).

suggScales(Class, Accidental, Qual,_, StartNote):-
    Qual=='minor',
    scale(Class, Accidental, ScaleNotes),
    random(X),
    minWriteScale(ScaleNotes, StartNote, X*10).

/*Possible dominant scales */

suggScales(Class, Accidental, Qual, ExtensionList, StartNote):-
    Qual=='dominant',
    member([flat|' ninth'], ExtensionList),
    scale(Class, Accidental, ScaleNotes),
    random(X),
    dom9bWriteScale(ScaleNotes, StartNote, X*10).

suggScales(Class, Accidental, Qual, ExtensionList, StartNote):-
    Qual=='dominant',
    member([' '|' fourth'], ExtensionList),
    scale(Class, Accidental, ScaleNotes),
    random(X),
    domSusWriteScale(ScaleNotes, StartNote, X*10).

suggScales(Class, Accidental, Qual, ExtensionList, StartNote):-
    Qual=='dominant',
    member([flat|' sixth'], ExtensionList),
    scale(Class, Accidental, ScaleNotes),
    random(X),
    dom6bWriteScale(ScaleNotes, StartNote, X*10).

suggScales(Class, Accidental, Qual, ExtensionList, StartNote):-
    Qual=='dominant',
    member([' '|' aug'], ExtensionList),
    scale(Class, Accidental, ScaleNotes),
    random(X),
    domAugWriteScale(ScaleNotes, StartNote, X*10).

suggScales(Class, Accidental, Qual, ExtensionList, StartNote):-
    Qual=='dominant',
    member([sharp|' ninth'], ExtensionList),
    scale(Class, Accidental, ScaleNotes),
    random(X),
    domSharpNinthWriteScale(ScaleNotes, StartNote, X*10).

suggScales(Class, Accidental, Qual, ExtensionList, StartNote):-
    Qual=='dominant',
    member([sharp|' fourth'], ExtensionList),
    scale(Class, Accidental, ScaleNotes),
    random(X),
    domSharpFourthWriteScale(ScaleNotes, StartNote, X*10).

suggScales(Class, Accidental, Qual, _, StartNote):-
    Qual=='dominant',
    scale(Class, Accidental, ScaleNotes),
    random(X),
    domWriteScale(ScaleNotes, StartNote, X*10).

/* Write scale functors.  These map to the scale functors in the 
   createScale.pl file.  startSc is the functor in the startScale.pl 
   file that tells the scale which note to start on, and also writes
   the scale.  The write scale functors are currently controlled by
   a random number generator in the suggScales functor.  This random
   number means that whenever there is more than one possible scale, 
   the scale displayed is picked randomly. /*

/* Dominant write scale funtions. */

domSharpFourthWriteScale(ScaleNotes, StartNote, _):-
    write('Lydian Dominant'), lydDom(ScaleNotes, LydDomScale),
    startSc(LydDomScale, StartNote), nl.

domSharpNinthWriteScale(ScaleNotes, StartNote, _):-
    write('Diminished Whole Tone '), dimWholeTone(ScaleNotes, Scale),
    startSc(Scale, StartNote), nl.

domAugWriteScale(ScaleNotes, StartNote, _):-
    write('Whole Tone'), wholeTone(ScaleNotes, WTScale),
    startSc(WTScale, StartNote), nl.

dom6bWriteScale(ScaleNotes, StartNote, _):-
    write('Hindu '), hindu(ScaleNotes, HindScale),
    startSc(HindScale, StartNote), nl.

domSusWriteScale(ScaleNotes, StartNote, X):-
    X=<3, write('Dominant, avoid third '), dom(ScaleNotes, DomScale),
    startSc(DomScale, StartNote), nl.

domSusWriteScale(ScaleNotes, StartNote, X):-
    X=<6, write('Pent '), susPent(ScaleNotes, PentScale),
    startSc(PentScale, StartNote),nl.

domSusWriteScale(ScaleNotes, StartNote, _):-
    write('Bebop '), bebopDom(ScaleNotes, BopScale),
    startSc(BopScale, StartNote),nl.

dom9bWriteScale(ScaleNotes, StartNote, X):-
    X=<5, write('Spanish or Jewish: '), spanish(ScaleNotes, SpanScale),
    startSc(SpanScale, StartNote), nl.

dom9bWriteScale(ScaleNotes, StartNote, _):-
    write('Diminished'), dimH(ScaleNotes, DimScale),
    startSc(DimScale, StartNote), nl.

domWriteScale(ScaleNotes, StartNote, X):-
    X=<3, write('Dominant'), dom(ScaleNotes, DomScale),
    startSc(DomScale, StartNote), nl.

domWriteScale(ScaleNotes, StartNote, X):-
    X=<5, write('Pent   '), pent(ScaleNotes, PentScale),
    startSc(PentScale, StartNote), nl.

domWriteScale(ScaleNotes, StartNote, X):-
    X=<8, write('Bebop Dominant'), bebopDom(ScaleNotes, BDomScale),
    startSc(BDomScale, StartNote), nl.

domWriteScale(ScaleNotes, StartNote, _):-
    write('Blues'), blues(ScaleNotes, BluesScale),
    startSc(BluesScale, StartNote), nl.

/* Major write scale functors */

maj7WriteScale(ScaleNotes, StartNote, X):-
    X=<5, write('Major  '), startSc(ScaleNotes, StartNote),
    nl.

maj7WriteScale(ScaleNotes, StartNote, _):-
    write('Bebop  '), bebopMaj(ScaleNotes, BScale),
    startSc(BScale, StartNote), nl.

maj45WriteScale(ScaleNotes, StartNote, _):-
    write('Lydian Augmented'), lydAug(ScaleNotes, LydAugScale),
    startSc(LydAugScale, StartNote), nl.

maj6WriteScale(ScaleNotes, StartNote, _):-
    write('Harmonic Major'), harmMaj(ScaleNotes, HarmScale),
    startSc(HarmScale, StartNote).

majSusWriteScale(ScaleNotes, StartNote, _):-
    write('Lydian '), lyd(ScaleNotes, LydScale),
    startSc(LydScale, StartNote),nl.

majWriteScale(ScaleNotes, StartNote, X):-
    X=<2, write('Major  '), startSc(ScaleNotes, StartNote),
    nl.

majWriteScale(ScaleNotes, StartNote, X):-
    X=<4, write('Pent   '), pent(ScaleNotes, PentScale),
    startSc(PentScale, StartNote), nl.

majWriteScale(ScaleNotes, StartNote, X):-
    X=<5, write('Augmented   '), aug(ScaleNotes, AugScale),
    startSc(AugScale, StartNote), nl.

majWriteScale(ScaleNotes, StartNote, X):-
    X=<7, write('Diminished (begin with H step): '), dimH(ScaleNotes, DimScale),
    startSc(DimScale, StartNote), nl.

majWriteScale(ScaleNotes, StartNote, X):-
    X=<8, write('6th mode of Harmonic Minor: '), 
    harmMin6Mode(ScaleNotes, Scale),
    startSc(Scale, StartNote), nl.

majWriteScale(ScaleNotes, StartNote, _):-
    write('Blues  '), blues(ScaleNotes, BluesScale),
    startSc(BluesScale, StartNote), nl.

/* Minor write scale functors */

min9bWriteScale(ScaleNotes, StartNote, _):-
    write('Phrygian '), phyr(ScaleNotes, PhyrScale),
    startSc(PhyrScale, StartNote), nl.

min6bWriteScale(ScaleNotes, StartNote, _):-
    write('Pure or Natural Minor '), pure(ScaleNotes, PureScale),
    startSc(PureScale, StartNote), nl.

minSharp7WriteScale(ScaleNotes, StartNote, X):-
    X=<3, 
    write('Melodic Minor '), melMin(ScaleNotes, MelMinScale),
    startSc(MelMinScale, StartNote), nl.

minSharp7WriteScale(ScaleNotes, StartNote, X):-
    X=<6,
    write('Bebop Minor 2'), bebopMin2(ScaleNotes, Bop2Scale),
    startSc(Bop2Scale, StartNote), nl.

minSharp7WriteScale(ScaleNotes, StartNote, _):-
    write('Harmonic Minor'), harmMin(ScaleNotes, HarmMinScale),
    startSc(HarmMinScale, StartNote), nl.

minWriteScale(ScaleNotes, StartNote, X):-
    X=<2,
    write('Dorian           '), dor(ScaleNotes, DorScale),
    startSc(DorScale, StartNote), nl.
    
minWriteScale(ScaleNotes, StartNote, X):-
    X=<4,
    write('Bebop Minor      '), bebopMin(ScaleNotes, BMin),
    startSc(BMin, StartNote), nl.
   
minWriteScale(ScaleNotes, StartNote, X):-
    X=<6,
    write('Diminished  '), dimW(ScaleNotes, DimMin),
    startSc(DimMin, StartNote), nl.

minWriteScale(ScaleNotes, StartNote, X):-
    X=<8,
    write('Blues            '), blues(ScaleNotes, BluesScale),
    startSc(BluesScale, StartNote),nl.
    
minWriteScale(ScaleNotes, StartNote, _):-
    write('Pentatonic Minor '), pentMin(ScaleNotes, PentMin),
    startSc(PentMin, StartNote), nl.

/* Locrian write scale functors */

locWriteScale(ScaleNotes, StartNote, X):-
    X=<5,
    write('Bebop      '), bebopLoc(ScaleNotes, BLocScale),
    startSc(BLocScale, StartNote), nl.

locWriteScale(ScaleNotes, StartNote, _):-
    write('Locrian    '), loc(ScaleNotes, LocScale),
    startSc(LocScale, StartNote), nl.

locSharp2WriteScale(ScaleNotes, StartNote, X):-
    X=<5, write('Bebop Locrian '), bebopLoc(ScaleNotes, BlocScale),
    startSc(BlocScale, StartNote), nl.

locSharp2WriteScale(ScaleNotes, StartNote, _):-
    write('Locrian #2 '), locSharp2(ScaleNotes, Loc2Scale),
    startSc(Loc2Scale, StartNote), nl.
