/* Katy Perdue
   Created June 27, 2003
   Last Modified July 1, 2003 */

:-ensure_loaded(chords).
:-ensure_loaded(parse).


maxList([], 0).
maxList([First|Rest], First):-
    maxList(Rest, RestMax),
    max(First, RestMax, First).
maxList([First|Rest], RestMax):-
    maxList(Rest, RestMax),
    max(First, RestMax, RestMax).



max(X, Y, X):- X>=Y.
max(X, Y, Y):- Y>X.


/* cor (short for corresponding) takes in a list of chords and a list
   of notes and outputs a list as a list of [chord, [corresponding notes]].
   Notes should be entered in the [Note, Duration] format.  */

/* cor(ChordList, NoteList, ChordsWNotes):-
    nl, cor(ChordList, NoteList, [], ChordsWNotes). */

cor(_, [], []):-!.
cor([FirstChord|ChordList], NoteList, [[FirstChord, OkNotes]|ChordsWNotes]):-
    parse(FirstChord, Dur, _, _, _, _),
    convertChord(Dur, DurTimes6),
    oneMeasure(DurTimes6, NoteList, OkNotes, LeftOverNotes),
    cor(ChordList, LeftOverNotes, ChordsWNotes).

/* convertChord is largely superfluous, but it will make it easier
   if we ever decide to break measures up into different chunks */

convertChord(ChordInQuarters, ChordTimes6):-
    ChordTimes6 is *(6, ChordInQuarters).

/* oneMeasure takes in a duration and a list of notes, and takes as
   many notes as fit in the duration.  This also breaks up notes that 
   are too long- the output (assuming there are enough notes in the list)
   will always be the same length as the chord durations. */

oneMeasure(ChordDur, NoteList, OkNotes, LeftOverNotes):-
    oneMeasure(ChordDur, NoteList, [], OkNotes, LeftOverNotes), !.

oneMeasure(0, NotesLeft, X, Y, NotesLeft):- reverse(X, Y), !.
oneMeasure(_, [],  X, Y, []):- reverse(X, Y), !.
oneMeasure(ChordDur, [FirstNote|NoteList], OkNotes, EndList, LeftOverNotes):-
    remove(FirstNote, _, [FirstNoteDur]),
    NewChordDur is -(ChordDur, FirstNoteDur),
    NewChordDur >= 0,
    oneMeasure(NewChordDur, NoteList, [FirstNote|OkNotes], 
	       EndList, LeftOverNotes), !.
oneMeasure(ChordDur, [FirstNote|NoteList], OkNotes, EndList, LeftOverNotes):-
    remove(FirstNote, NoteName, [FirstNoteDur]),
    NewChordDur is -(ChordDur, FirstNoteDur),
    NewChordDur< 0,
    NewNoteDur is -(FirstNoteDur, ChordDur),
    oneMeasure(0, [[NoteName, NewNoteDur]|NoteList], 
	       [[NoteName, ChordDur]|OkNotes], EndList, LeftOverNotes).

% functors not currently in use


/* miniParse was set up as a parser for notes.  It is not terribly useful
   at the present time (as I decided on a different format), but it could
   be modified in the future to parse notes in a useful way. */

miniParse(Note, Duration, Name, Accidental):-
    name(Note, [Dur, Name1, Accidental1]),
    Duration is -(Dur, 48),
    class(Name1, Name),
    sharpOrFlat(Accidental1, Accidental).
miniParse(Note, Duration, Name, ' '):-
    name(Note, [Dur, Name1]),
    Duration is -(Dur, 48),
    class(Name1, Name).

/* measures breaks a whole string of notes into same-size chunks.
   This turns out to be less useful than I thought... */

measures(ChordDur, NoteList, MeasureList):-
    measures(ChordDur, NoteList, [], MeasureList).

measures(_, [], X, Y):- reverse(X,Y),!.
measures(ChordDur, NoteList, MeasureList, End):-
    oneMeasure(ChordDur, NoteList, OkNotes, LeftOverNotes),
    measures(ChordDur, LeftOverNotes, [OkNotes|MeasureList], End).


