/* Katy Perdue
   Created June 14, 2003
   Last modified June 25, 2003 */

:-ensure_loaded(class).
:-ensure_loaded(parse).
:-ensure_loaded(createScale).
:-ensure_loaded(guido).
:-ensure_loaded(library(random)).

/*writeNotes takes in pitches described by their pitch (in 'D_flat' format), 
  octave, duration numerator (DurNum), and duration denominator (DurDem).  
  This style is designed to be compatible with the guido parsing system. */

writeNotes(Something):-write('['), writeNotes(Something, yes).
writeNotes([], yes):-write(']').
writeNotes([PitchClass, Octave, DurNum, DurDem|Rest], yes):-
    inputTrans(PitchClass, GuidoClass), 
    write(GuidoClass), write(Octave), write('*'), write(DurNum),
    write('/'), write(DurDem), write(' '), writeNotes(Rest, yes).

/* pick4 is a little predicate that makes 4 beats of 8th notes, according to 
   the goal-note method (Berg), with specific placement of goal notes (and lesser
   goals).  It's mostly for fun.  One problem is that it can suggest rests, but 
   it also specifies an octave for the rests, which guido doesn't like, so the
   rests (specified by '_') have to be manually edited when they are chosen. */

pick4(Note, Accidental, Type):-
    getScale([Note, Accidental, Type], ReturnScale),
    allDegrees(ReturnScale, Root, Second, Third, Fourth, 
	       Fifth, Sixth, Seventh),
    random(X), pickGoal(10*X, Third, Seventh, ChosenNote1),
    random(Y), pickGoal(10*Y, Third, Seventh, ChosenNote2),
    random(N1), pickLesserGoal(N1*10, Root, Fifth, LessGoal1),
    random(N0), pickLesserGoal(N0*10, Root, Fifth, LessGoal2),
    random(N2), pickNote(N2*8, Root, Second, Third, Fourth,
			 Fifth, Sixth, Seventh, Rand1),
    random(N3), pickNote(N3*8, Root, Second, Third, Fourth,
                         Fifth, Sixth, Seventh, Rand2),
    random(N4), pickNote(N4*8, Root, Second, Third, Fourth,
                         Fifth, Sixth, Seventh, Rand3),
    writeNotes([ChosenNote1, 1, 1,8, Rand1, 1, 1, 8, LessGoal1, 1, 1, 8, 
		Rand2,1,1,8, Rand3, 1, 1, 8, ChosenNote2, 1, 1, 8,
		LessGoal2, 1, 1, 8, Third, 1, 1, 8]).


allDegrees([Root, Second, Third, Fourth, Fifth, Sixth, Seventh],
	   Root, Second, Third, Fourth, Fifth, Sixth, Seventh).


pickGoal(X,  Third, _, ChosenNote):-
    X<5, ChosenNote = Third.
pickGoal(_, _, Seventh, ChosenNote):-
    ChosenNote = Seventh.

pickLesserGoal(X, Root, _, ChosenNote):- X<5, ChosenNote=Root.
pickLesserGoal(_, _, Fifth, ChosenNote):- ChosenNote=Fifth. 

pickNote(X,Root, _, _, _, _, _, _, ChosenNote):- X<1,ChosenNote=Root.
pickNote(X,_, Second, _, _, _, _, _, ChosenNote):- X<2,ChosenNote=Second.
pickNote(X,_, _,Third, _, _, _, _, ChosenNote):- X<3,ChosenNote=Third.
pickNote(X,_, _, _, Fourth, _, _, _, ChosenNote):- X<4,ChosenNote=Fourth.
pickNote(X,_, _, _, _, Fifth, _, _, ChosenNote):- X<5,ChosenNote=Fifth.
pickNote(X,_, _, _, _, _, Sixth, _, ChosenNote):- X<6,ChosenNote=Sixth.
pickNote(X,_, _, _, _, _, _, Seventh, ChosenNote):- X<7,ChosenNote=Seventh.
pickNote(_,_, _, _, _, _, _, _, ChosenNote):- ChosenNote='_'.

