/* Katy Perdue
   created on or before June 1, 2003 */


:-ensure_loaded(class).
:-ensure_loaded(scales).


/* parse2 both parses the user-entered chord, and suggests scales that go
   with that chord.  The functor is set up to accept either a single chord
   or a list of chords.  The auxilary functors are located in parse.pl */

parse([Chord|[]], Dur, Class, Accidental, Qual, Ext):-
    parse(Chord, Dur, Class, Accidental, Qual, Ext), !.

parse([Chord|List], _, _, _,_,_):-
    parse(Chord, _, _, _, _, _), nl,
    parse(List, _,_, _, _, _).

parse(Chord, Dur, Class, Accidental, Qual, ExtensionList):-
    write('Chord: '), write(Chord), nl,
    name(Chord, [PreDur|ChordAtoms]),
    Dur is -(PreDur,48),
    class(ChordAtoms, Class),
    sharpOrFlat(ChordAtoms, [Q, U, A| L ],  Accidental),
    quality(Q, U, A, Qual),
    findall([ExtSign|Ext], extensions(L, [ExtSign|Ext]), ExtensionList). 



/* extensions is a form of extension (from the parse.pl file) designed
   to handle more than one extension (or none at all). */

extensions([Num|Rest], [ExtSign|Extension]):-
    remove(Rest, Acc, _),
    extension(Num, Extension),
    sharpOrFlat(Acc, ExtSign).

extensions([_|Rest], [ExtSign|Extension]):-
    remove(Rest, _, TheRest),
    extensions(TheRest, [ExtSign|Extension]). 


/*
parse([Chord|[]]):- parse(Chord).
parse([Chord|List]):- parse(Chord), nl,  parse(List).


parse(Chord):-
    name(Chord, [_|ChordAtoms]),
    class(Chord, Class), nl,
    write('Class: '), write(Class), nl,
    sharpOrFlat(ChordAtoms, [Q, U, A, L,I,T ],  Design),
    write('Accidental: '), write(Design), nl,
    quality(Q, U, A, Qual),
    write('Quality: '), write(Qual), nl,
    extension(L, Extension),
    sharpOrFlat(I, ExtenSign),
    write('Extension: '), write(ExtenSign), write(Extension), nl,
    Duration is -(T,48),
    write('Duration: '), write(Duration), write(' quarter notes'). 

parse([Chord|[]], Class, Accidental, Qual, Ext, ExtSign, Dur):-
    parse(Chord, Class, Accidental, Qual, Ext, ExtSign, Dur).

parse([Chord|List], _, _, _,_,_, _):-
    parse(Chord, _, _, _, _, _, _), nl,
    parse(List, _,_, _, _, _,_).

parse(Chord, Class, Accidental, Qual, Ext, ExtSign, Dur):-
    name(Chord, [_|ChordAtoms]),
    class(Chord, Class),
    sharpOrFlat(ChordAtoms, [Q, U, A, L,I,T ],  Accidental),
    quality(Q, U, A, Qual),
    extension(L, Ext),
    sharpOrFlat(I, ExtSign),
    Dur is -(T,48).
 */

extension(Ext, Extension):-
    Ext==55, Extension= ' seventh'.
extension(Ext, Extension):-
    Ext==57, Extension=' ninth'.
extension(Ext, Extension):-
    Ext==49, Extension=' eleventh'.
extension(Ext, Extension):-
    Ext==50, Extension= ' second'.
extension(Ext, Extension):-
    Ext==51, Extension= ' thirteenth'.
extension(Ext, Extension):-
    Ext==52, Extension= ' fourth'.
extension(Ext, Extension):-
    Ext==53, Extension= ' fifth'.
extension(Ext, Extension):-
    Ext==54, Extension= ' sixth'.
extension(Ext, Extension):-
    Ext==43, Extension= ' aug'.
extension(Ext, Extension):-
    Ext==95, Extension=' none'.

quality(M, A, J, Qual):-
    M==109 , A==97 , J==106, Qual ='major'.
quality(M,I,N, Qual):-
    M==109, I==105, N==110, Qual = 'minor'.
quality(D, O, M, Qual):-
    D==100, O==111 , M==109, Qual = 'dominant'.
quality(D, I, M, Qual):-
    D==100, I==105, M==109, Qual ='diminished'.
quality(L, O, C, Qual):-
    L==108, O==111, C==99,  Qual= 'locrian or half diminished'.

sharpOrFlat(Sign, flat ):-
    Sign==98, !.
sharpOrFlat(Sign, sharp):-
    Sign==35, !.
sharpOrFlat(Sign, ' ' ):-
    Sign==95, !.
sharpOrFlat([], ' ').

sharpOrFlat([_|Qual], Rest , ' flat' ):-
    remove(Qual, Sign, Rest),
    Sign==98, !.

sharpOrFlat([_|Qual], Rest,' sharp'  ):-
    remove(Qual, Sign, Rest),
    Sign==35, !.

sharpOrFlat([_|Qual], Rest, '_' ):-
    remove(Qual, Sign, Rest),
    Sign==95, !.

remove([Sign|Rest], Sign, Rest).
