/* Katy Perdue
   Created June 12, 2003
   Last Modified June 25, 2003 */

:-ensure_loaded(createScale).
:-ensure_loaded(scales).
:-ensure_loaded(equals).

/* Structures are implemented, at least in that sharedNote/
   allShared take in lists of scales in the 
   chord(Note, Accidental, Scale) form.  The abstraction is not
   continued down as far as it should be (get scale takes in a 
   list of arguments rather than just one), but it is functional.
   chord is probably not a good name for the scale structure, but
   it works for now.  */


/* SharedNote determines if a note is shared between two keys.*/

sharedNote(_, []).
sharedNote(Note, [chord(Note1, Acc1, Type1)|Rest]):-
    getScale([Note1, Acc1, Type1], ReturnScale1),
    member(Note, ReturnScale1),
    sharedNote(Note, Rest).

/* allShared finds a list of all the notes that the entered scales
   have in common.  findall is a built-in predicate.  */

allShared(Note, [Chord1|Rest]):-
    findall(Note2, sharedNote(Note2, [Chord1|Rest]), Note2),
    reduce(Note2, Note).

/* reduce eliminates repeated notes in the all shared list.  */

reduce([First, Sec|Rest], [First|ReducedList]):-
    equals(First, Sec),
    reduce(Rest, ReducedList), !. 
reduce([First, Sec|Rest], [First|ReducedList]):-
    reduce([Sec|Rest], ReducedList),!.
reduce(First, First):-!.

/* The classic member function, using equals to ensure enharmonic notes
   are treated as equal.*/
member(X, [Z|_]):- equals(X,Z).
member(X, [_|R]) :- member(X, R).


/* getScale takes in a note, accidental, and type of scale, and returns
   the notes of the scale specified.  */
getScale([Note, Accidental, Type], ReturnScale):-
    scale(Note, Accidental, Scale),
    switch([Type, Scale, ReturnScale]).


/* switch uses the built in predicate =..  .  This takes the first element
   of the list and uses it as the functor.  For example, 
   switch([note, accidental, duration]) calls note(accidental, duration). */
switch(X):-
    Goal=..X, call(Goal).
